@@ -1,10 +1,11 @@
 ;;; elmo-localnews.el -- Local News Spool Interface for ELMO.
 
-;; Copyright 1998,1999,2000 OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
-;;                          Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 1998,1999,2000 OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
+;; Copyright (C) 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author:  OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
-;; Time-stamp: <2000-01-24 12:20:07 teranisi>
+;;	Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, net news
 
 ;; This file is part of ELMO (Elisp Library for Message Orchestration).
 
@@ -122,12 +123,14 @@
    (elmo-localdir-get-msg-filename spec number loc-alist)))
 
 (defalias 'elmo-localnews-sync-number-alist 'elmo-generic-sync-number-alist)
-(defalias 'elmo-localnews-list-folder-unread 
+(defalias 'elmo-localnews-list-folder-unread
   'elmo-generic-list-folder-unread)
 (defalias 'elmo-localnews-list-folder-important
   'elmo-generic-list-folder-important)
 (defalias 'elmo-localnews-commit 'elmo-generic-commit)
+(defalias 'elmo-localnews-folder-diff 'elmo-generic-folder-diff)
 
-(provide 'elmo-localnews)
+(require 'product)
+(product-provide (provide 'elmo-localnews) (require 'elmo-version))
 
 ;;; elmo-localnews.el ends here
