@@ -1,6 +1,21 @@
+;;; bbdb-wl.el -- BBDB interface to Wanderlust
+
+;; Copyright (C) 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, news, database
+
+;;; Commentary:
 ;;
+;;  Insert the following lines in your ~/.wl
+;; 
+;;  (require 'bbdb-wl)
+;;  (bbdb-wl-setup)
+
+;;; Code:
+;;
+
 ;; bbdb setup.
-;;
 (eval-when-compile
   (require 'mime-setup)
   (require 'elmo-vars)
@@ -37,7 +52,7 @@
   (add-hook 'wl-mail-setup-hook
 	    (function
 	     (lambda ()
-;	       (local-set-key "\M-\t" 'bbdb-complete-name)
+;;;	       (local-set-key "\M-\t" 'bbdb-complete-name)
 	       (define-key (current-local-map) "\M-\t" 'bbdb-complete-name)
 	       ))))
 
@@ -61,8 +76,8 @@
 (defun bbdb-wl-show-bbdb-buffer ()
   (save-selected-window
     (if (get-buffer-window bbdb-buffer-name)
-	nil  
-      (let ((mes-win (get-buffer-window 
+	nil
+      (let ((mes-win (get-buffer-window
 		      (save-excursion
 			(if (buffer-live-p  wl-current-summary-buffer)
 			    (set-buffer wl-current-summary-buffer))
@@ -75,7 +90,7 @@
 			window-min-height 1)
 		     (- (window-height mes-win)
 			(max window-min-height
-			     (1+ bbdb-pop-up-target-lines))))))    
+			     (1+ bbdb-pop-up-target-lines))))))
 	  (split-window mes-win (if (> size 0) size window-min-height)))
 	;; goto the bottom of the two...
 	(select-window (next-window))
@@ -84,11 +99,11 @@
 	  (switch-to-buffer (get-buffer-create bbdb-buffer-name)))))))
 
 (defun bbdb-wl-from-func (string)
-  "A candidate for wl-summary-from-func..."
+  "A candidate From field STRING.  For `wl-summary-from-func'."
   (let ((hit (bbdb-search-simple nil (wl-address-header-extract-address
 				      string)))
 	first-name last-name from-str)
-    (if hit 
+    (if hit
 	(progn
 	  (setq first-name (aref hit 0))
 	  (setq last-name (aref hit 1))
@@ -103,16 +118,16 @@
       string)))
 
 (defun bbdb-wl-update-record (&optional offer-to-create)
-  "Returns the record corresponding to the current WL message, 
-creating or modifying it as necessary.  A record will be created if 
+  "Returns the record corresponding to the current WL message,
+creating or modifying it as necessary.  A record will be created if
 bbdb/mail-auto-create-p is non-nil, or if OFFER-TO-CREATE is true and
 the user confirms the creation."
   (save-excursion
     (if bbdb-use-pop-up
 	(bbdb-wl-pop-up-bbdb-buffer offer-to-create)
-      (let ((key 
+      (let ((key
 	     (save-excursion
-	       (set-buffer 
+	       (set-buffer
 		(save-excursion
 		  (if (buffer-live-p wl-current-summary-buffer)
 		      (set-buffer wl-current-summary-buffer))
@@ -149,10 +164,10 @@ the user confirms the creation."
 			 offer-to-create))))))))))
 
 (defun bbdb-wl-annotate-sender (string)
-  "Add a line to the end of the Notes field of the BBDB record 
+  "Add a line to the end of the Notes field of the BBDB record
 corresponding to the sender of this message."
   (interactive (list (if bbdb-readonly-p
-			 (error "The Insidious Big Brother Database is read-only.")
+			 (error "The Insidious Big Brother Database is read-only")
 		       (read-string "Comments: "))))
   (set-buffer (wl-message-get-original-buffer))
   (bbdb-annotate-notes (bbdb-wl-update-record t) string))
@@ -171,7 +186,7 @@ of the BBDB record corresponding to the sender of this message."
 
 (defun bbdb-wl-show-sender ()
   "Display the contents of the BBDB for the sender of this message.
-This buffer will be in bbdb-mode, with associated keybindings."
+This buffer will be in `bbdb-mode', with associated keybindings."
   (interactive)
   (wl-summary-redisplay)
   (set-buffer (wl-message-get-original-buffer))
@@ -181,7 +196,7 @@ This buffer will be in bbdb-mode, with associated keybindings."
 	(progn
 	  (bbdb-wl-pop-up-bbdb-buffer)
 	  (bbdb-display-records (list record)))
-      (error "unperson"))
+      (error "Unperson"))
     (setq bbdb-win (get-buffer-window (get-buffer bbdb-buffer-name)))
     (and bbdb-win
 	 (select-window bbdb-win))))
@@ -191,8 +206,8 @@ This buffer will be in bbdb-mode, with associated keybindings."
   "Make the *BBDB* buffer be displayed along with the WL window(s),
 displaying the record corresponding to the sender of the current message."
   (if (get-buffer-window bbdb-buffer-name)
-      nil  
-    (let ((mes-win (get-buffer-window 
+      nil
+    (let ((mes-win (get-buffer-window
 		    (save-excursion
 		      (if (buffer-live-p  wl-current-summary-buffer)
 			  (set-buffer wl-current-summary-buffer))
@@ -206,7 +221,7 @@ displaying the record corresponding to the sender of the current message."
 		      window-min-height 1)
 		   (- (window-height mes-win)
 		      (max window-min-height
-			   (1+ bbdb-pop-up-target-lines))))))    
+			   (1+ bbdb-pop-up-target-lines))))))
 	(split-window mes-win (if (> size 0) size window-min-height)))
       ;; goto the bottom of the two...
       (select-window (next-window))
@@ -235,20 +250,27 @@ displaying the record corresponding to the sender of the current message."
 ;;; @ bbdb-extract-field-value -- stolen from tm-bbdb.
 ;;;
 (and (not (fboundp 'bbdb-extract-field-value-internal))
-     (not (fboundp 'tm:bbdb-extract-field-value)) ;; tm-bbdb
-;;   (not (fboundp 'PLEASE_REPLACE_WITH_SEMI-BASED_MIME-BBDB)) ;; mime-bbdb
+;;;  (not (fboundp 'PLEASE_REPLACE_WITH_SEMI-BASED_MIME-BBDB)) ;; mime-bbdb
     (progn
-      ;; (require 'bbdb-hooks) ; not provided.
-      ;; (or (fboundp 'bbdb-extract-field-value) ; defined as autoload
+;;;   (require 'bbdb-hooks) ; not provided.
+;;;   (or (fboundp 'bbdb-extract-field-value) ; defined as autoload
       (or (fboundp 'bbdb-header-start)
           (load "bbdb-hooks"))
       (fset 'bbdb-extract-field-value-internal
-            (symbol-function 'bbdb-extract-field-value))
+	    (cond
+	     ((fboundp 'tm:bbdb-extract-field-value)
+	      (symbol-function 'tm:bbdb-extract-field-value))
+	     (t (symbol-function 'bbdb-extract-field-value))))
       (defun bbdb-extract-field-value (field)
 	(let ((value (bbdb-extract-field-value-internal field)))
-	  (and value
-	       (eword-decode-string value))))
+	  (with-temp-buffer ; to keep raw buffer unibyte.
+	    (elmo-set-buffer-multibyte
+	     default-enable-multibyte-characters)
+	    (and value
+		 (eword-decode-string value)))))
       ))
 
 
 (provide 'bbdb-wl)
+
+;;; bbdb-wl.el ends here
