@@ -1,10 +1,11 @@
 ;;; wl.el -- Wanderlust bootstrap.
 
-;; Copyright 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 1998,1999,2000 Masahiro MURATA <muse@ba2.so-net.ne.jp>
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;;	Masahiro MURATA <muse@ba2.so-net.ne.jp>
 ;; Keywords: mail, net news
-;; Time-stamp: <2000-04-07 09:15:33 teranisi>
 
 ;; This file is part of Wanderlust (Yet Another Message Interface on Emacsen).
 
@@ -25,10 +26,10 @@
 ;;
 
 ;;; Commentary:
-;; 
+;;
 
 ;;; Code:
-;; 
+;;
 
 (require 'elmo2)
 ;; from x-face.el
@@ -42,12 +43,16 @@
 
 (require 'wl-vars)
 (require 'wl-util)
+(require 'wl-version)
 
-(if wl-on-xemacs
-    (require 'wl-xmas)
-  (if wl-on-nemacs
-      (require 'wl-nemacs)
-    (require 'wl-mule)))
+(cond (wl-on-xemacs
+       (require 'wl-xmas))
+      (wl-on-emacs21
+       (require 'wl-e21))
+      (wl-on-nemacs
+       (require 'wl-nemacs))
+      (t
+       (require 'wl-mule)))
 
 (provide 'wl) ; circular dependency
 (require 'wl-folder)
@@ -62,7 +67,7 @@
 (require 'wl-demo)
 (require 'wl-highlight)
 
-(eval-when-compile 
+(eval-when-compile
   (require 'smtp)
   (require 'wl-score)
   (unless wl-on-nemacs
@@ -78,14 +83,10 @@
   (when make-alist
     (wl-make-plugged-alist))
   ;; Plug status.
-  (setq elmo-plugged (setq wl-plugged (elmo-plugged-p)))
-  (setq wl-plug-state-indicator
-	(if wl-plugged
-	    wl-plug-state-indicator-on
-	  wl-plug-state-indicator-off))
+  (setq elmo-plugged (setq wl-plugged (elmo-plugged-p))
+	wl-modeline-plug-status wl-plugged)
   (if wl-plugged
-      (wl-toggle-plugged t 'flush))
-  (force-mode-line-update t))
+      (wl-toggle-plugged t 'flush)))
 
 (defun wl-toggle-plugged (&optional arg queue-flush-only)
   (interactive)
@@ -98,21 +99,21 @@
       (setq wl-plugged nil))
      (t (setq wl-plugged (null wl-plugged))))
     (elmo-set-plugged wl-plugged))
-  (setq elmo-plugged wl-plugged)
+  (setq elmo-plugged wl-plugged
+	wl-modeline-plug-status wl-plugged)
   (save-excursion
-    (mapcar 
-     (function 
-      (lambda (x)
-	(set-buffer x)
+    (let ((summaries (wl-collect-summary)))
+      (while summaries
+	(set-buffer (pop summaries))
 	(wl-summary-msgdb-save)
 	;; msgdb is saved, but cache is not saved yet.
-	(wl-summary-set-message-modified)))
-     (wl-collect-summary)))
+	(wl-summary-set-message-modified))))
+  (setq wl-biff-check-folders-running nil)
   (if wl-plugged
       (progn
 	;; flush queue!!
-	(setq wl-plug-state-indicator wl-plug-state-indicator-on)
 	(elmo-dop-queue-flush)
+	(unless queue-flush-only (wl-biff-start))
 	(if (and wl-draft-enable-queuing
 		 wl-auto-flush-queue)
 	    (wl-draft-queue-flush))
@@ -125,7 +126,7 @@
 		  (wl-summary-flush-pending-append-operations seen-list))
 	    (elmo-msgdb-seen-save msgdb-dir seen-list)))
 	(run-hooks 'wl-plugged-hook))
-    (setq wl-plug-state-indicator wl-plug-state-indicator-off)
+    (wl-biff-stop)
     (run-hooks 'wl-unplugged-hook))
   (force-mode-line-update t))
 
@@ -141,7 +142,7 @@
   '(("Queuing" . wl-draft-enable-queuing)
     ("AutoFlushQueue" . wl-auto-flush-queue)
     ("DisconnectedOperation" . elmo-enable-disconnected-operation)))
- 
+
 (defvar wl-plugged-buf-name "Plugged")
 (defvar wl-plugged-mode-map nil)
 (defvar wl-plugged-alist nil)
@@ -151,9 +152,6 @@
 (defvar wl-plugged-dop-queue-alist nil)
 (defvar wl-plugged-alist-modified nil)
 
-(defvar wl-plugged-glyph nil)
-(defvar wl-unplugged-glyph nil)
-
 (defvar wl-plugged-mode-menu-spec
   '("Plugged"
     ["Toggle plugged" wl-plugged-toggle t]
@@ -210,8 +208,7 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
   (setq major-mode 'wl-plugged-mode)
   (setq mode-name "Plugged")
   (easy-menu-add wl-plugged-mode-menu)
-  (when wl-show-plug-status-on-modeline 
-    (setq mode-line-format (wl-make-modeline)))
+  (wl-mode-line-buffer-identification)
   (setq wl-plugged-switch wl-plugged)
   (setq wl-plugged-alist-modified nil)
   (setq buffer-read-only t)
@@ -515,13 +512,10 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 	    (elmo-set-plugged plugged nil nil nil alist)))
 	  ;; redraw
 	  (wl-plugged-redrawing wl-plugged-alist)
-	  ;; change wl-plug-state-indicator
+	  ;; show plugged status in modeline
 	  (let ((elmo-plugged wl-plugged-switch))
-	    (setq wl-plugged-switch (elmo-plugged-p))
-	    (setq wl-plug-state-indicator
-		  (if wl-plugged-switch
-		      wl-plug-state-indicator-on
-		    wl-plug-state-indicator-off))
+	    (setq wl-plugged-switch (elmo-plugged-p)
+		  wl-modeline-plug-status wl-plugged-switch)
 	    (force-mode-line-update t))))))
     (setq wl-plugged-alist-modified t)
     (goto-char cur-point)))
@@ -539,11 +533,8 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
     (wl-plugged-redrawing wl-plugged-alist)
     (goto-char cur-point)
     (setq wl-plugged-alist-modified t)
-    ;; change wl-plug-state-indicator
-    (setq wl-plug-state-indicator
-	  (if wl-plugged-switch
-	      wl-plug-state-indicator-on
-	    wl-plug-state-indicator-off))
+    ;; show plugged status in modeline
+    (setq wl-modeline-plug-status wl-plugged-switch)
     (force-mode-line-update t)))
 
 (defun wl-plugged-exit ()
@@ -618,30 +609,26 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
   (let (summary-buf)
     (save-excursion
       (let ((summaries (wl-collect-summary)))
-	(mapcar 
-	 (function
-	  (lambda (x)
-	    (set-buffer x)
-	    (unless keep-summary
-	      (wl-summary-cleanup-temp-marks))
-	    (wl-summary-save-status keep-summary)
-	    (unless keep-summary
-	      (kill-buffer x))))
-	 summaries))))
-  (wl-refile-alist-save
-   wl-refile-alist-file-name wl-refile-alist)
-  (wl-refile-alist-save
-   wl-refile-msgid-alist-file-name wl-refile-msgid-alist)
+	(while summaries
+	  (set-buffer (car summaries))
+	  (unless keep-summary
+	    (wl-summary-cleanup-temp-marks))
+	  (wl-summary-save-status keep-summary)
+	  (unless keep-summary
+	    (kill-buffer (car summaries)))
+	  (setq summaries (cdr summaries))))))
+  (wl-refile-alist-save)
   (wl-folder-info-save)
   (and (featurep 'wl-fldmgr) (wl-fldmgr-exit))
   (wl-crosspost-alist-save)
-  (message "Saving summary and folder status...done."))
+  (message "Saving summary and folder status...done"))
 
 (defun wl-exit ()
   (interactive)
   (when (or (not wl-interactive-exit)
-	    (y-or-n-p "Quit Wanderlust?"))
+	    (y-or-n-p "Quit Wanderlust? "))
     (elmo-quit)
+    (wl-biff-stop)
     (run-hooks 'wl-exit-hook)
     (wl-save-status)
     (wl-folder-cleanup-variables)
@@ -652,7 +639,8 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 			(list (format "%s\\(:.*\\)?"
 				      (default-value 'wl-message-buf-name))
 			      wl-original-buf-name
-			      wl-folder-buffer-name)
+			      wl-folder-buffer-name
+			      wl-plugged-buf-name)
 			"\\|")))
     (elmo-buffer-cache-clean-up)
     (if (fboundp 'mmelmo-cleanup-entity-buffers)
@@ -686,11 +674,11 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 	      (setq elmo-use-semi nil))
 	    ;; defined above.
 	    (wl-mime-setup)
-	    (fset 'wl-summary-from-func-internal 
+	    (fset 'wl-summary-from-func-internal
 		  (symbol-value 'wl-summary-from-func))
-	    (fset 'wl-summary-subject-func-internal 
+	    (fset 'wl-summary-subject-func-internal
 		  (symbol-value 'wl-summary-subject-func))
-	    (fset 'wl-summary-subject-filter-func-internal 
+	    (fset 'wl-summary-subject-filter-func-internal
 		  (symbol-value 'wl-summary-subject-filter-func))
 	    (setq elmo-no-from wl-summary-no-from-message)
 	    (setq elmo-no-subject wl-summary-no-subject-message)
@@ -698,11 +686,14 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 	    (progn
 	      (message "Checking environment...")
 	      (wl-check-environment arg)
-	      (message "Checking environment...done.")))
+	      (message "Checking environment...done")))
 	(if demo-buf
 	    (kill-buffer demo-buf))
 	(if succeed
 	    (setq wl-init t))
+	;; This hook may contain the functions `wl-plugged-init-icons' and
+	;; `wl-biff-init-icons' for reasons of system internal to accord
+	;; facilities for the Emacs variants.
 	(run-hooks 'wl-init-hook)))))
 
 (defun wl-check-environment (no-check-folder)
@@ -710,39 +701,49 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
     (require 'mime-setup))
   (unless wl-from
     (error "Please set `wl-from'"))
+  ;; Message-ID
   (unless (string-match "[^.]\\.[^.]" (or wl-message-id-domain
 					  (if wl-local-domain
- 					      (concat (system-name)
+					      (concat (system-name)
 						      "." wl-local-domain)
- 					    (system-name))))
+					    (system-name))))
     (error "Please set `wl-local-domain' to get valid FQDN"))
+  (if (string-match "@" (or wl-message-id-domain
+			    (if wl-local-domain
+				(concat (system-name)
+					"." wl-local-domain)
+			      (system-name))))
+      (error "Please remove `@' from `wl-message-id-domain'"))
+  ;; folders
   (when (not no-check-folder)
     (if (not (eq (elmo-folder-get-type wl-draft-folder) 'localdir))
 	(error "%s is not allowed for draft folder" wl-draft-folder))
     (unless (elmo-folder-exists-p wl-draft-folder)
-      (if (y-or-n-p 
-	   (format "Draft Folder %s does not exist, create it?" 
+      (if (y-or-n-p
+	   (format "Draft Folder %s does not exist, create it? "
 		   wl-draft-folder))
 	  (elmo-create-folder wl-draft-folder)
 	(error "Draft Folder is not created")))
     (if (and wl-draft-enable-queuing
 	     (not (elmo-folder-exists-p wl-queue-folder)))
-	(if (y-or-n-p 
-	     (format "Queue Folder %s does not exist, create it?" 
+	(if (y-or-n-p
+	     (format "Queue Folder %s does not exist, create it? "
 		     wl-queue-folder))
 	    (elmo-create-folder wl-queue-folder)
-	  (error "Queue Folder is not created")))
+	  (error "Queue Folder is not created"))))
+  (when (not (eq no-check-folder 'wl-draft))
     (unless (elmo-folder-exists-p wl-trash-folder)
-      (if (y-or-n-p 
-	   (format "Trash Folder %s does not exist, create it?" 
+      (if (y-or-n-p
+	   (format "Trash Folder %s does not exist, create it? "
 		   wl-trash-folder))
 	  (elmo-create-folder wl-trash-folder)
 	(error "Trash Folder is not created")))
     (unless (elmo-folder-exists-p elmo-lost+found-folder)
       (elmo-create-folder elmo-lost+found-folder)))
+  ;; tmp dir
   (unless (file-exists-p wl-tmp-dir)
-    (if (y-or-n-p 
-	 (format "Temp directory (to save multipart) %s does not exist, create it now?" 
+    (if (y-or-n-p
+	 (format "Temp directory (to save multipart) %s does not exist, create it now? "
 		 wl-tmp-dir))
 	(make-directory wl-tmp-dir)
       (error "Temp directory is not created"))))
@@ -750,15 +751,20 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 ;;;###autoload
 (defun wl (&optional arg)
   "Start Wanderlust -- Yet Another Message Interface On Emacsen.
-If prefix argument is specified, folder checkings are skipped."
+If ARG (prefix argument) is specified, folder checkings are skipped."
   (interactive "P")
-  (unless wl-init
-    (wl-load-profile))
+  (or wl-init (wl-load-profile))
   (unwind-protect
       (wl-init arg)
-    (let ((make (wl-folder arg)))
-      (wl-plugged-init make)))
-  (run-hooks 'wl-hook))
+    (wl-plugged-init (wl-folder arg))
+    (sit-for 0))
+  (unwind-protect
+      (unless arg
+	(run-hooks 'wl-auto-check-folder-pre-hook)
+	(wl-folder-auto-check)
+	(run-hooks 'wl-auto-check-folder-hook))
+    (unless arg (wl-biff-start))
+    (run-hooks 'wl-hook)))
 
 ;; Define some autoload functions WL might use.
 (eval-and-compile
@@ -807,7 +813,8 @@ If prefix argument is specified, folder checkings are skipped."
 
 ;; for backward compatibility
 (defalias 'wl-summary-from-func-petname 'wl-summary-default-from)
- 
-(provide 'wl)
+
+(require 'product)
+(product-provide (provide 'wl) (require 'wl-version))
 
 ;;; wl.el ends here
